/*
 * DBCollectionIndexSource.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Ghostar, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Data;
using TopCoder.Data.ConnectionFactory;

namespace TopCoder.Document.Index.Persistence.DB
{
    /// <summary>
    /// Implements the <see cref="ICollectionIndexSource"/> to provide the collection index sources
    /// in a database.
    /// </summary>
    ///
    /// <remarks>
    /// It holds properties for connection parameters, either passed in or loaded from the configuration manager.
    /// It only works with the Document table in the database, for loading in document indices.
    /// Note that all changes are in a transaction, and <see cref="Save"/> is used to commit it.
    /// This class is mutable.
    /// </remarks>
    ///
    /// <example>
    /// Here's a sample configuration file for this class: <code><![CDATA[
    /// <?xml version="1.0" encoding="UTF-8"?>
    /// <!DOCTYPE ConfigManager SYSTEM "cm.dtd">
    ///
    /// <ConfigManager>
    ///     <!-- a sample valid configuration -->
    ///     <namespace name="TopCoder.Document.Index.Persistence.DB">
    ///         <!-- the db connection name (req) -->
    ///         <property name="DBConnectionName">
    ///             <value>InformixOdbc</value>
    ///         </property>
    ///     </namespace>
    /// </ConfigManager>
    /// ]]></code>
    /// Here's a sample code: <code><![CDATA[
    /// ICollectionIndexSource dbSource = null;
    /// try
    /// {
    ///     // load the collection from the database
    ///     dbSource = new DBCollectionIndexSource();
    ///     IParameters parameters = new DBPersistenceParameters(DbConnectionName);
    ///     dbSource.Load(parameters);
    ///
    ///     // enumerate through indices
    ///     IDictionaryEnumerator enumerator = dbSource.GetEnumerator();
    ///     int count = 0;
    ///     while (enumerator.MoveNext())
    ///     {
    ///         count++;
    ///     }
    ///
    ///     // add a new index
    ///     IParameters documentParams = new DBDocumentIndexParameters(DbConnectionName, 1);
    ///     IndexInfo info = new IndexInfo(typeof(DBDocumentIndexSource).Assembly.GetName().Name,
    ///         typeof(DBDocumentIndexSource).FullName, documentParams);
    ///     dbSource.Add("New document", info);
    ///
    ///     // add using indexer
    ///     dbSource["New document"] = info;
    ///
    ///     // save the changes
    ///     dbSource.Save();
    ///
    ///     // check index exists
    ///     bool exists = dbSource.Contains("New document");
    ///
    ///     // remove the index
    ///     dbSource.Remove("New document");
    ///
    ///     // use indexer
    ///     IndexInfo retrieved = dbSource["Document 2"];
    ///
    ///     // clear all indices
    ///     dbSource.Clear();
    ///
    ///     // save the changes
    ///     dbSource.Save();
    /// }
    /// finally
    /// {
    ///     if (dbSource != null)
    ///     {
    ///         // dispose db resources
    ///         ((IDisposable) dbSource).Dispose();
    ///     }
    /// }
    /// ]]></code>
    /// </example>
    ///
    /// <threadsafety static="false" instance="false"/>
    public class DBCollectionIndexSource : ICollectionIndexSource, IDisposable
    {
        /// <summary>
        /// The config key for the name of the db connection in the Connection Factory component.
        /// </summary>
        private const string DBConnectionNameConfig = "DBConnectionName";

        /// <summary>
        /// The type to populate IndexInfo (used in DbLoad).
        /// </summary>
        private static readonly Type DBDocumentIndexSourceType = typeof(DBDocumentIndexSource);

        /// <summary>
        /// SQL statement to insert into Document table.
        /// Compare_Options and Culture will have a default value of "None" and "en-US".
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlInsertDocument = new SqlStatement(string.Format(
            "INSERT INTO Document (Document_Name, Compare_Options, Culture) "
            + "VALUES (?, '{0}', '{1}')", Helper.DefaultCompareOptions.ToString(), Helper.DefaultCulture),
            "Document_Name");

        /// <summary>
        /// SQL statement to delete from the Document table.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlDeleteDocument = new SqlStatement(
            "DELETE FROM Document WHERE Document_Name = ?", "Document_Name");

        /// <summary>
        /// SQL statement to select all rows from the Document table.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlSelectDocument = new SqlStatement(
            "SELECT Document_Id, Document_Name FROM Document ORDER BY Document_Id");

        /// <summary>
        /// SQL statement to delete all rows from the Document table.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlTruncateDocument = new SqlStatement(
            "DELETE FROM Document");

        /// <summary>
        /// SQL statement to delete all rows from the Word table.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlTruncateWord = new SqlStatement(
            "DELETE FROM Word");

        /// <summary>
        /// SQL statement to delete all rows from the Location table.
        /// This is disposed in Dispose().
        /// </summary>
        private SqlStatement sqlTruncateLocation = new SqlStatement(
            "DELETE FROM Location");

        /// <summary>
        /// This variable holds the configuration information for this class,
        /// including the name of the connection loaded.
        /// This value is initialized to a new DBPersistenceParameters instance
        /// in the constructor, using values read from the namespace given to the constructor.
        /// It can also be changed via the Load method.
        /// </summary>
        private DBPersistenceParameters parameters;

        /// <summary>
        /// This connection is the actual connection that is used to talk to the database for updates.
        /// This value is set using the Connection Factory component in the constructors.
        /// The System.Data.Odbc classes should be used when executing commands against this connection.
        /// as we need to be able to connect to an Informix database, as specified in the RS.
        /// The connection will be closed as soon as the transaction is committed/rolled-back.
        /// </summary>
        private IDbConnection connection;

        /// <summary>
        /// All updates are added to this transaction, and they are committed in the Save() method.
        /// This is lazily initialized to a new IDbTransaction implementation using
        /// IDbConnection.BeginTransaction() method whenever needed.
        /// After transaction is committed/rolled-back, this will be initialized to null
        /// and connection will be closed to ensure no db resource is left hanging.
        /// </summary>
        private IDbTransaction transaction;

        /// <summary>
        /// This member variable holds the loaded names and document indices loaded from the database
        /// that this instance is working against.
        /// The keys in this dictionary are the names, given by the user, of each document index,
        /// and the values are the IndexInfo instances, created in the Load method,
        /// that can be accessed by the user.
        /// This collection is accessed using the accessors in this class,
        /// and it is loaded in the Load methods.
        /// </summary>
        private IDictionary indices = new SortedList();

        /// <summary>
        /// Tracks whether Dispose has been called.
        /// </summary>
        private bool isDisposed = false;

        /// <summary>
        /// Initializes a new instance of the <see cref="DBCollectionIndexSource"/> class.
        /// </summary>
        ///
        /// <remarks>
        /// This constructor creates a new <see cref="DBCollectionIndexSource"/> instance
        /// using the default namespace <see cref="DBDocumentIndexSource.DefaultNamespace"/> constant.
        /// The instance will initially be blank.
        /// The <see cref="Load"/> method should be used to load the actual data in
        /// for this class.
        /// </remarks>
        ///
        /// <exception cref="DBPersistenceConfigurationException">
        /// The default namespace doesn't contain the required configuration parameters.
        /// </exception>
        public DBCollectionIndexSource()
            : this(DBDocumentIndexSource.DefaultNamespace)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DBCollectionIndexSource"/> class
        /// based on configuration parameters read from the given namespace.
        /// </summary>
        ///
        /// <remarks>
        /// Note that <see cref="Load"/> method should be used to load the actual data in
        /// for this class.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// Namespace to load configuration from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="nameSpace"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is an empty string.
        /// </exception>
        /// <exception cref="DBPersistenceConfigurationException">
        /// The namespace doesn't contain the required configuration parameters.
        /// </exception>
        public DBCollectionIndexSource(string nameSpace)
        {
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace", "'nameSpace' cannot be null");
            }
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("'nameSpace' cannot be an empty trimmed string", "nameSpace");
            }

            try
            {
                // get connection name from configuration file (req)
                string connName = Helper.GetCMValue(nameSpace, DBConnectionNameConfig);

                // initialize parameters and create the DB connection
                parameters = new DBPersistenceParameters(connName);
                connection = ConnectionManager.Instance.CreateDbConnection(connName, new Hashtable());
            }
            catch (DBPersistenceConfigurationException)
            {
                // re-throw
                throw;
            }
            catch (Exception e)
            {
                throw new DBPersistenceConfigurationException(string.Format(
                    "Error configuring from namespace '{0}'", nameSpace), e);
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="IndexInfo"/> for the given collection index name.
        /// </summary>
        ///
        /// <param name="indexName">
        /// Name of the index to return or set the <see cref="IndexInfo"/> for.
        /// </param>
        ///
        /// <value>
        /// The <see cref="IndexInfo"/> for the given collection index name or
        /// null if no <see cref="IndexInfo"/> for the given name exists in the collection.
        /// The word will be trimmed before stored.
        /// </value>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="indexName"/> is a null reference or setting to a null value.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="indexName"/> is an empty trimmed string or
        /// the <paramref name="indexInfo"/> is invalid (eg. the assembly/class name must be
        /// <see cref="DBDocumentIndexSource"/> and the parameter must be
        /// of type <see cref="DBDocumentIndexParameters"/>.
        /// </exception>
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while setting the document indices.
        /// </exception>
        public IndexInfo this[string indexName]
        {
            get
            {
                if (indexName == null)
                {
                    throw new ArgumentNullException("indexName", "'indexName' cannot be null");
                }
                if (indexName.Trim().Length == 0)
                {
                    throw new ArgumentException("'indexName' cannot be an empty trimmed string", "indexName");
                }

                return (IndexInfo) indices[indexName];
            }
            set
            {
                if (indexName == null)
                {
                    throw new ArgumentNullException("indexName", "'indexName' cannot be null");
                }
                string key = indexName.Trim();
                if (key.Length == 0)
                {
                    throw new ArgumentException("'indexName' cannot be an empty trimmed string", "indexName");
                }
                if (value == null)
                {
                    throw new ArgumentNullException("indexInfo", "'indexInfo' cannot be null");
                }
                Helper.ValidateIndexInfo(value, DBDocumentIndexSourceType, typeof(DBDocumentIndexParameters));

                // insert into database (if document name doesn't exist yet)
                if (!Contains(key))
                {
                    DbInsertDocument(key);
                    indices[key] = value;
                }
            }
        }

        /// <summary>
        /// Adds the given document into the database (Document table).
        /// </summary>
        ///
        /// <param name="indexName">
        /// The name of the index to add.
        /// </param>
        /// <param name="indexInfo">
        /// The <see cref="IndexInfo"/> to add with the given name.
        /// </param>
        ///
        /// <remarks>
        /// The word will be trimmed before stored.
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="indexName"/> is an empty trimmed string or
        /// if a value for the specified index name already exists or
        /// the <paramref name="indexInfo"/> is invalid (eg. the assembly/class name must be
        /// <see cref="DBDocumentIndexSource"/> and the parameter must be
        /// of type <see cref="DBDocumentIndexParameters"/>.
        /// </exception>
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while inserting the document into the database.
        /// </exception>
        public void Add(string indexName, IndexInfo indexInfo)
        {
            if (indexName == null)
            {
                throw new ArgumentNullException("indexName", "'indexName' cannot be null");
            }
            string key = indexName.Trim();
            if (key.Length == 0)
            {
                throw new ArgumentException("'indexName' cannot be an empty trimmed string", "indexName");
            }
            if (indexInfo == null)
            {
                throw new ArgumentNullException("indexInfo", "'indexInfo' cannot be null");
            }
            Helper.ValidateIndexInfo(indexInfo, DBDocumentIndexSourceType, typeof(DBDocumentIndexParameters));

            // insert into database (if document name doesn't exist yet)
            if (Contains(key))
            {
                throw new ArgumentException("'indexName' already exists", "indexName");
            }
            else
            {
                DbInsertDocument(key);
                indices[key] = indexInfo;
            }
        }

        /// <summary>
        /// Removes the given document from the database (Document table)
        /// and all related locations for the corresponding document (Location table).
        /// </summary>
        ///
        /// <remarks>
        /// Nothing will occur if the specified name doesn't exist.
        /// </remarks>
        ///
        /// <param name="indexName">
        /// Name of the index to remove.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="indexName"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="indexName"/> is an empty trimmed string.
        /// </exception>
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while deleting the document from the database.
        /// </exception>
        public void Remove(string indexName)
        {
            if (indexName == null)
            {
                throw new ArgumentNullException("indexName", "'indexName' cannot be null");
            }
            if (indexName.Trim().Length == 0)
            {
                throw new ArgumentException("'indexName' cannot be an empty trimmed string", "indexName");
            }

            DbDeleteDocument(indexName);
            indices.Remove(indexName);
        }

        /// <summary>
        /// Returns the enumerator for the indices collection, which can be used by the user
        /// to enumerate through all document indices in this collection.
        /// </summary>
        ///
        /// <returns>
        /// The enumerator for all <see cref="IndexInfo"/> in the collection.
        /// </returns>
        public IDictionaryEnumerator GetEnumerator()
        {
            return indices.GetEnumerator();
        }

        /// <summary>
        /// Determines whether the document name exists.
        /// </summary>
        ///
        /// <param name="indexName">
        /// The index name to look for.
        /// </param>
        ///
        /// <returns>
        /// true if the collection contains the given index; otherwise, false.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="indexName"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="indexName"/> is an empty trimmed string.
        /// </exception>
        public bool Contains(string indexName)
        {
            if (indexName == null)
            {
                throw new ArgumentNullException("indexName", "'indexName' cannot be null");
            }
            if (indexName.Trim().Length == 0)
            {
                throw new ArgumentException("'indexName' cannot be an empty trimmed string", "indexName");
            }

            return indices.Contains(indexName);
        }

        /// <summary>
        /// Clears all document indices. This will remove all rows from the Document,
        /// Location, and Word tables.
        /// </summary>
        ///
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while clearing all documents from the database.
        /// </exception>
        public void Clear()
        {
            DbClear();
            indices.Clear();
        }

        /// <summary>
        /// Loads all data from the Document table.
        /// </summary>
        ///
        /// <param name="parameters">
        /// The <see cref="DBPersistenceParameters"/> containing the connection name to use.
        /// </param>
        ///
        /// <remarks>
        /// <para>
        /// Each Document row from the database is parsed, with the name being the key,
        /// and a new <see cref="IndexInfo"/> being created with
        /// a <see cref="DBDocumentIndexParameters"/> instance created
        /// with this class' connection parameters, and
        /// the DocumentId value taken for each row from the database.
        /// </para>
        /// <para>
        /// The params value given is a <see cref="DBPersistenceParameters"/> instance
        /// that is used to set the connection parameters of this class.
        /// A new connection is retrieved using those values using Connection Factory component.
        /// </para>
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="parameters"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="parameters"/> is not a <see cref="DBPersistenceParameters"/> instance.
        /// </exception>
        /// <exception cref="DBPersistenceConfigurationException">
        /// The given parameter contains invalid values.
        /// </exception>
        /// <exception cref="LoadIndexDataException">
        /// Any errors occur while loading the data from the database.
        /// </exception>
        public void Load(IParameters parameters)
        {
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters", "'parameters' cannot be null");
            }
            if (!(parameters is DBPersistenceParameters))
            {
                throw new ArgumentException("'parameters' must be of DBPersistenceParameters type", "parameters");
            }

            // create a new db connection from the parameters
            try
            {
                // but first rollback any pending transaction and close previous connection
                RollsbackTransaction();

                string connName = ((DBPersistenceParameters) parameters).ConnectionName;
                connection = ConnectionManager.Instance.CreateDbConnection(connName, new Hashtable());

                // reinitialize parameters
                this.parameters = (DBPersistenceParameters) parameters;
            }
            catch (Exception e)
            {
                throw new DBPersistenceConfigurationException(
                    "Error creating database connection", e);
            }

            // loads all document name and populates indices IDictionary
            DbLoad();
        }

        /// <summary>
        /// Commits all changes to the database.
        /// </summary>
        ///
        /// <exception cref="SaveIndexDataException">
        /// An error occurs while committing the transaction.
        /// </exception>
        public void Save()
        {
            try
            {
                if (transaction != null)
                {
                    // commit, reset transaction to null, and close connection
                    transaction.Commit();
                    transaction = null;
                    connection.Close();
                }
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, "Error committing transaction");
            }
        }

        /// <summary>
        /// Disposes the various db resources.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            // tell GC not to run finalizer
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Finalizers which is called by the garbage collector.
        /// </summary>
        ~DBCollectionIndexSource()
        {
            // Do not re-create Dispose clean-up code here.
            // Calling Dispose(false) is optimal in terms of
            // readability and maintainability.
            Dispose(false);
        }

        /// <summary>
        /// Implements the IDisposable pattern as specified in the MSDN documentation.
        /// </summary>
        ///
        /// <param name="disposing">
        /// true if this is called by user (managed resource can be referenced),
        /// false if this is called by garbage collector (don't touch managed resource).
        /// </param>
        private void Dispose(bool disposing)
        {
            if (!isDisposed)
            {
                // can only reference managed resource if user invokes the dispose
                if (disposing)
                {
                    // roll back transaction and closes connection
                    RollsbackTransaction();

                    // dispose all SqlStatement
                    sqlInsertDocument.Dispose();
                    sqlDeleteDocument.Dispose();
                    sqlSelectDocument.Dispose();

                    sqlTruncateLocation.Dispose();
                    sqlTruncateWord.Dispose();
                    sqlTruncateDocument.Dispose();
                }
            }
            isDisposed = true;
        }

        /// <summary>
        /// Inserts into the Document table.
        /// </summary>
        ///
        /// <param name="indexName">
        /// The collection index name.
        /// </param>
        ///
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while inserting the document.
        /// </exception>
        private void DbInsertDocument(string indexName)
        {
            try
            {
                InitTransaction();
                // INSERT INTO Document (Document_Name, Compare_Options, Culture)
                // VALUES (?, "None", "en-US");
                sqlInsertDocument.ExecuteNonQuery(connection, transaction, indexName);
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, string.Format(
                    "Error inserting document '{0}'", indexName));
            }
        }

        /// <summary>
        /// Deletes from the Document table and all rows from the Location table
        /// for the corresponding document id.
        /// </summary>
        ///
        /// <param name="indexName">
        /// The collection index name.
        /// </param>
        ///
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while deleting the document.
        /// </exception>
        private void DbDeleteDocument(string indexName)
        {
            try
            {
                InitTransaction();
                // DELETE FROM Document WHERE Document_Name = ?
                sqlDeleteDocument.ExecuteNonQuery(connection, transaction, indexName);
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, string.Format(
                    "Error deleting document '{0}'", indexName));
            }
        }

        /// <summary>
        /// Truncates all rows from the Document, Location, and Word tables.
        /// </summary>
        ///
        /// <exception cref="SaveIndexDataException">
        /// Any error occurs while truncating the tables.
        /// </exception>
        private void DbClear()
        {
            try
            {
                InitTransaction();
                // DELETE FROM Location/Word/Document
                sqlTruncateLocation.ExecuteNonQuery(connection, transaction);
                sqlTruncateWord.ExecuteNonQuery(connection, transaction);
                sqlTruncateDocument.ExecuteNonQuery(connection, transaction);
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, "Error clearing database schema");
            }
        }

        /// <summary>
        /// Loads all document from the Document table, and build indices IDictionary.
        /// </summary>
        ///
        /// <exception cref="LoadIndexDataException">
        /// Any error occurs while loading the tables.
        /// </exception>
        private void DbLoad()
        {
            try
            {
                InitTransaction();
                indices.Clear();

                // SELECT Document_Id, Document_Name FROM Document ORDER BY Document_Id
                using (IDataReader reader = sqlSelectDocument.ExecuteReader(connection, transaction))
                {
                    while (reader.Read())
                    {
                        // prepare IndexInfo and add to indices IDictionary
                        IParameters idxParam = new DBDocumentIndexParameters(
                            parameters.ConnectionName, reader.GetInt64(0));
                        IndexInfo idxInfo = new IndexInfo(
                            DBDocumentIndexSourceType.Assembly.FullName,
                            DBDocumentIndexSourceType.FullName,
                            idxParam);
                        indices[reader.GetString(1)] = idxInfo;
                    }
                }
            }
            catch (Exception e)
            {
                throw new LoadIndexDataException(parameters, e, "Error loading data from Document table");
            }
            finally
            {
                // since no data is modified, we can safely roll back the opened transaction
                // and closes connection
                RollsbackTransaction();
            }
        }

        /// <summary>
        /// Lazily begins transaction if one is not started yet.
        /// This method should be called before any db operations are performed
        /// although only the first call will actually open the connection and starts transaction.
        /// </summary>
        private void InitTransaction()
        {
            if (transaction == null)
            {
                connection.Open();
                transaction = connection.BeginTransaction();
            }
        }

        /// <summary>
        /// Rolls back any pending transaction and closes the db connection.
        /// </summary>
        private void RollsbackTransaction()
        {
            if (transaction != null)
            {
                transaction.Rollback();
                transaction = null;
            }
            if (connection.State != ConnectionState.Closed)
            {
                connection.Close();
            }
        }
    }
}
