/*
 * DBPersistenceParameters.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Ghostar, TCSDEVELOPER
 */

using System;

namespace TopCoder.Document.Index.Persistence.DB
{
    /// <summary>
    /// This class holds options used to initialize both <see cref="DBDocumentIndexSource"/>
    /// and <see cref="DBCollectionIndexSource"/>.
    /// </summary>
    ///
    /// <remarks>
    /// The connection name property is used to get a connection from the ConnectionFactory component.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class DBPersistenceParameters : IParameters
    {
        /// <summary>
        /// This member variable holds the connection name, used to create a connection from the Connection Factory.
        /// This value is set in the constructor and accessed by the user using the ConnectionName property.
        /// </summary>
        private readonly string connectionName;

        /// <summary>
        /// Gets the connection name which is used to create database connection
        /// using the Connection Factory component.
        /// </summary>
        ///
        /// <value>
        /// The connection name.
        /// </value>
        public string ConnectionName
        {
            get
            {
                return connectionName;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DBPersistenceParameters"/>.
        /// </summary>
        ///
        /// <param name="connectionName">
        /// Name of the connection to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="connectionName"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="connectionName"/> is an empty trimmed string.
        /// </exception>
        public DBPersistenceParameters(string connectionName)
        {
            if (connectionName == null)
            {
                throw new ArgumentNullException("connectionName", "'connectioName' cannot be null");
            }
            if (connectionName.Trim().Length == 0)
            {
                throw new ArgumentException("'connectionName' cannot be an empty trimmed string", "connectionName");
            }
            this.connectionName = connectionName;
        }

        /// <summary>
        /// Serves as a hash function for a particular type,
        /// suitable for use in hashing algorithms and data structures like a hash table.
        /// </summary>
        ///
        /// <returns>
        /// A hash code of this instance.
        /// </returns>
        public override int GetHashCode()
        {
            return connectionName.GetHashCode();
        }

        /// <summary>
        /// Determines whether two <see cref="DBPersistenceParameters"/> instances are equal.
        /// </summary>
        ///
        /// <param name="obj">
        /// Object to compare to.
        /// </param>
        ///
        /// <returns>
        /// true if the specified object is equal to this instance; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            if (!(obj is DBPersistenceParameters))
            {
                return false;
            }
            return (ConnectionName == ((DBPersistenceParameters) obj).ConnectionName);
        }
    }
}
