/*
 * Helper.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Globalization;
using System.IO;
using System.Xml;
using System.Xml.Schema;
using TopCoder.Util.ConfigurationManager;
using TopCoder.Document.Index.Persistence.DB;

namespace TopCoder.Document.Index.Persistence
{
    /// <summary>
    /// Common helper containing various static constants and helper methods.
    /// </summary>
    ///
    /// <threadsafety static="true" instance="true"/>
    internal sealed class Helper
    {
        /// <summary>
        /// The number of indentation space for the xml file.
        /// </summary>
        internal const int XmlIndentation = 4;

        /// <summary>
        /// The default culture info.
        /// </summary>
        internal static string DefaultCulture = "en-US";

        /// <summary>
        /// The default CompareOptions.
        /// </summary>
        internal static CompareOptions DefaultCompareOptions = CompareOptions.None;

        /// <summary>
        /// A private constructor to prevents this class from being instantiated.
        /// </summary>
        private Helper()
        {
        }

        /// <summary>
        /// Validates a file path string.
        /// The file path cannot be null, an empty trimmed string,
        /// and cannot be an invalid file path (malformed or points to a directory),
        /// but the file doesn't need to exist.
        /// </summary>
        ///
        /// <param name="filePath">
        /// The string representing the file path to validate.
        /// </param>
        /// <param name="paramName">
        /// The parameter name, used for the exception message.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="filePath"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="filePath"/> is an empty trimmed string or is invalid
        /// (either it is malformed or it points to a directory).
        /// </exception>
        internal static void ValidateFilePath(string filePath, string paramName)
        {
            if (filePath == null)
            {
                throw new ArgumentNullException(paramName, string.Format(
                    "'{0}' cannot be null", paramName));
            }
            if (filePath.Trim().Length == 0)
            {
                throw new ArgumentException(string.Format(
                    "'{0}' cannot be an empty trimmed string", paramName), paramName);
            }

            // must be a valid path (not malformed and not a directory)
            try
            {
                FileAttributes attr = File.GetAttributes(filePath);

                // cannot be a directory
                if ((attr & FileAttributes.Directory) != 0)
                {
                    throw new ArgumentException(string.Format(
                        "'{0}' cannot be a directory", filePath), paramName);
                }
            }
            catch (FileNotFoundException)
            {
                // file not found is ok
            }
            catch (ArgumentException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                throw new ArgumentException(string.Format(
                    "'{0}' is not a valid file path", filePath), paramName, e);
            }
        }

        /// <summary>
        /// Validates that the given IndexInfo has its assembly/class name
        /// points to the given type and the parameter must be of type paramType.
        /// </summary>
        ///
        /// <param name="indexInfo">
        /// The IndexInfo to validate.
        /// </param>
        /// <param name="type">
        /// The type which must be defined in the indexInfo.
        /// </param>
        /// <param name="paramType">
        /// The parameter type of the indexInfo.
        /// </param>
        ///
        /// <exception cref="ArgumentException">
        /// Thrown if the given IndexInfo is invalid.
        /// </exception>
        internal static void ValidateIndexInfo(IndexInfo indexInfo, Type type, Type paramType)
        {
            // compare with the simple name (indexInfo may contains full name)
            string asmName = type.Assembly.GetName().Name;
            if (!(indexInfo.AssemblyName.StartsWith(asmName)))
            {
                throw new ArgumentException(string.Format(
                    "'indexInfo.AssemblyName' must be {0}", asmName));
            }
            // compare the name only (classname may also be fully qualified with assembly name)
            if (!indexInfo.ClassName.StartsWith(type.FullName))
            {
                throw new ArgumentException(string.Format(
                    "'indexInfo.ClassName' must be {0}", type.FullName));
            }
            // Parameters must be of paramType
            if (!(paramType.IsAssignableFrom(indexInfo.Parameters.GetType())))
            {
                throw new ArgumentException(string.Format(
                    "'indexInfo.Parameters' must be of type {0}", paramType));
            }
        }

        /// <summary>
        /// Loads and validates filePath into an XmlDocument.
        /// </summary>
        ///
        /// <remarks>
        /// The file is loaded using <see cref="XmlValidatingReader"/> using a schema.
        /// The schema will provide xml structure validation.
        /// </remarks>
        ///
        /// <param name="filePath">
        /// The path to the xml file to load.
        /// </param>
        /// <param name="schema">
        /// The xml schema to validate against.
        /// </param>
        ///
        /// <returns>
        /// A validated <see cref="XmlDocument"/>.
        /// </returns>
        internal static XmlDocument LoadAndValidateXml(string filePath, string schema)
        {
            XmlValidatingReader reader = null;
            try
            {
                // create validating reader, load the XSD schema (to validate structure)
                XmlDocument doc = new XmlDocument();
                XmlTextReader txtReader = new XmlTextReader(filePath);
                reader = new XmlValidatingReader(txtReader);
                using (StringReader xmlSchema = new StringReader(schema))
                {
                    reader.Schemas.Add(XmlSchema.Read(xmlSchema, null));
                }

                doc.Load(reader);
                return doc;
            }
            finally
            {
                if (reader != null)
                {
                    // automatically close all underlying streams
                    reader.Close();
                }
            }
        }

        /// <summary>
        /// Gets a required value from the configuration manager.
        /// </summary>
        ///
        /// <param name="nameSpace">
        /// The namespace to load value from.
        /// </param>
        /// <param name="property">
        /// The property key.
        /// </param>
        ///
        /// <returns>
        /// The value in the specified property.
        /// </returns>
        ///
        /// <exception cref="DBPersistenceConfigurationException">
        /// The value doesn't exist or is an empty trimmed string.
        /// </exception>
        internal static string GetCMValue(string nameSpace, string property)
        {
            ConfigManager cm = ConfigManager.GetInstance();

            // get the value (required)
            string val = cm.GetValue(nameSpace, property);
            if ((val == null) || (val.Trim().Length == 0))
            {
                throw new DBPersistenceConfigurationException(string.Format(
                    "Missing required configuration '{0}'", val));
            }
            return val;
        }

        /// <summary>
        /// Determines whether object can be safely cast to long.
        /// </summary>
        ///
        /// <param name="o">
        /// The object to check.
        /// </param>
        ///
        /// <returns>
        /// true if object given is an integer/long/short/byte.
        /// </returns>
        internal static bool IsLong(object o)
        {
            return ((o is long) || (o is ulong)
                || (o is int) || (o is uint)
                || (o is short) || (o is ushort)
                || (o is sbyte) || (o is byte));
        }
    }
}
