/*
 * TestXmlCollectionIndexSource.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using System.Text;
using NUnit.Framework;
using TopCoder.Document.Index.Persistence.Tests;

namespace TopCoder.Document.Index.Persistence.Xml.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="XmlCollectionIndexSource"/> class.
    /// </summary>
    [TestFixture]
    public class TestXmlCollectionIndexSource : TestCommon
    {
        /// <summary>
        /// The <see cref="XmlCollectionIndexSource"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private XmlCollectionIndexSource source;

        /// <summary>
        /// The document collection index xml file for testing.
        /// </summary>
        private const string colFile = @"..\..\test_files\Collection.xml";

        /// <summary>
        /// The original (unmodified) document collection index xml file.
        /// </summary>
        private const string colOrigFile = @"..\..\test_files\Collection.orig.xml";

        /// <summary>
        /// The document index xml files for testing.
        /// </summary>
        private static string[] files ={
            @"..\..\test_files\Doc0.xml",
            @"..\..\test_files\Doc1.xml"};

        /// <summary>
        /// The assembly name for XmlDocumentIndexSource
        /// </summary>
        private static string assemblyName = typeof(XmlDocumentIndexSource).Assembly.GetName().Name;

        /// <summary>
        /// The class name for XmlDocumentIndexSource.
        /// </summary>
        private const string className = "TopCoder.Document.Index.Persistence.Xml.XmlDocumentIndexSource";

        /// <summary>
        /// Sample IndexInfo.
        /// Initialized in SetUp().
        /// </summary>
        private IndexInfo[] indexInfo;

        /// <summary>
        /// Initializes the <see cref="XmlCollectionIndexSource"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // prepare sample collection file
            TearDown();

            // prepare 2 sample IndexInfo
            indexInfo = new IndexInfo[2];
            indexInfo[0] = new IndexInfo(assemblyName, className,  new SimpleIndexParameters(files[0]));
            indexInfo[1] = new IndexInfo(assemblyName, className,  new SimpleIndexParameters(files[1]));

            // prepare an XmlCollectionIndexSource and adds two IndexInfo
            source = new XmlCollectionIndexSource(colFile);
            source.Add("Doc0", indexInfo[0]);
            source.Add("Doc1", indexInfo[1]);
        }

        /// <summary>
        /// Cleans up by replacing the collection file with the unmodified one.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            File.Copy(colOrigFile, colFile, true);
        }

        /// <summary>
        /// Tests constructor() accuracy and that FilePath is initially null
        /// and no index is stored (which is tested via GetEnumerator and indexer).
        /// </summary>
        [Test]
        public void TestCtor()
        {
            // prepare a blank XmlCollectionIndexSource
            source = new XmlCollectionIndexSource();
            Assert.AreEqual(null, source.FilePath, "Initially FilePath is null");

            // test via GetEnumerator
            IDictionaryEnumerator it = source.GetEnumerator();
            Assert.IsFalse(it.MoveNext(), "Nothing to enumerate");

            // test via this indexer
            Assert.IsNull(source["Doc0"], "Doc not exists");
            Assert.IsNull(source["Doc1"], "Doc1 not exists");
        }

        /// <summary>
        /// Tests constructor(String) accuracy and that FilePath is initialized properly,
        /// and no index is stored (which is tested via GetEnumerator and indexer) initially.
        /// </summary>
        [Test]
        public void TestCtorString()
        {
            // prepare a XmlCollectionIndexSource to initialize a file
            source = new XmlCollectionIndexSource(colFile);
            Assert.AreEqual(colFile, source.FilePath, "FilePath is initialized");

            // test via GetEnumerator
            IDictionaryEnumerator it = source.GetEnumerator();
            Assert.IsFalse(it.MoveNext(), "Nothing to enumerate");

            // test via this indexer
            Assert.IsNull(source["Doc0"], "Doc not exists");
            Assert.IsNull(source["Doc1"], "Doc1 not exists");
        }

        /// <summary>
        /// Tests constructor(String) given null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorStringNull()
        {
            new XmlCollectionIndexSource(null);
        }

        /// <summary>
        /// Tests constructor(String) given empty string argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringEmpty()
        {
            new XmlCollectionIndexSource(" ");
        }

        /// <summary>
        /// Tests constructor(String) given a path that points to a directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringDirectory()
        {
            new XmlCollectionIndexSource(@"..\..\test_files");
        }

        /// <summary>
        /// Tests constructor(String) given a path that points to the current directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringDirectoryCurrent()
        {
            new XmlCollectionIndexSource(".");
        }

        /// <summary>
        /// Tests constructor(String) given a path that points to the parent directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringDirectoryParent()
        {
            new XmlCollectionIndexSource("..");
        }

        /// <summary>
        /// Tests constructor(String) given a malformed path that contains a &gt; char.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringMalformedIllegalChar()
        {
            new XmlCollectionIndexSource(@"..\..\test_files\malformed>name");
        }

        /// <summary>
        /// Tests constructor(String) given a very long path (malformed).
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringMalformedLong()
        {
            // build a very long file name
            StringBuilder path = new StringBuilder();
            path.Append('a', 100000);
            new XmlCollectionIndexSource(path.ToString());
        }

        /// <summary>
        /// Tests SyncRoot accuracy, that it returns consistent object for locking.
        /// </summary>
        [Test]
        public void TestSyncRoot()
        {
            Assert.IsNotNull(source.SyncRoot, "A locking object is provided");
            Assert.AreSame(source.SyncRoot, source.SyncRoot,
                "Consistent object for locking is not returned");
        }

        /// <summary>
        /// Tests FilePath setter and getter accuracy.
        /// </summary>
        [Test]
        public void TestFilePath()
        {
            Assert.AreEqual(colFile, source.FilePath, "FilePath is initialized incorrectly");
            source.FilePath = files[0];
            Assert.AreEqual(files[0], source.FilePath, "Setting FilePath to a new value");
        }

        /// <summary>
        /// Tests setting FilePath to a null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestFilePathNull()
        {
            source.FilePath = null;
        }

        /// <summary>
        /// Tests setting FilePath to an empty string argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathEmpty()
        {
            source.FilePath = " ";
        }

        /// <summary>
        /// Tests setting FilePath to a directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathDirectory()
        {
            source.FilePath = @"..\..\test_files";
        }

        /// <summary>
        /// Tests setting FilePath to the current directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathDirectoryCurrent()
        {
            source.FilePath = ".";
        }

        /// <summary>
        /// Tests setting FilePath to the parent directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathDirectoryParent()
        {
            source.FilePath = "..";
        }

        /// <summary>
        /// Tests setting FilePath to a malformed path that contains a &gt; char.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathMalformedIllegalChar()
        {
            source.FilePath = @"..\..\test_files\malformed>name";
        }

        /// <summary>
        /// Tests setting FilePath to a very long path (malformed).
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestFilePathMalformedLong()
        {
            // build a very long file name
            StringBuilder path = new StringBuilder();
            path.Append('a', 100000);
            source.FilePath = path.ToString();
        }

        /// <summary>
        /// Tests get-indexer accuracy as initialized in SetUp().
        /// </summary>
        [Test]
        public void TestIndexerGet()
        {
            // 1st index info
            IndexInfo idx = source["Doc0"];
            Assert.AreEqual(assemblyName, idx.AssemblyName, "First AssemblyName is incorrect");
            Assert.AreEqual(className, idx.ClassName, "First ClassName is incorrect");
            Assert.AreEqual(files[0], ((SimpleIndexParameters) idx.Parameters).FileName,
                "First Parameter is incorrect");

            // 2nd index info
            idx = source["Doc1"];
            Assert.AreEqual(assemblyName, idx.AssemblyName, "First AssemblyName is incorrect");
            Assert.AreEqual(className, idx.ClassName, "First ClassName is incorrect");
            Assert.AreEqual(files[1], ((SimpleIndexParameters) idx.Parameters).FileName,
                "First Parameter is incorrect");

            Assert.IsNull(source["NotExists"], "Not exists");
        }

        /// <summary>
        /// Tests get-indexer given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerGetNull()
        {
            IndexInfo idx = source[null];
        }

        /// <summary>
        /// Tests get-indexer given empty index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerGetEmpty()
        {
            IndexInfo idx = source[" "];
        }

        /// <summary>
        /// Tests set-indexer accuracy by setting a new index info.
        /// </summary>
        [Test]
        public void TestIndexerSet()
        {
            // create a new index info
            IndexInfo idx = new IndexInfo(assemblyName, className, new SimpleIndexParameters("file"));
            source["new file"] = idx;

            IndexInfo assert = source["new file"];
            Assert.AreEqual(assemblyName, idx.AssemblyName, "AssemblyName is incorrect");
            Assert.AreEqual(className, idx.ClassName, "ClassName is incorrect");
            Assert.AreEqual("file", ((SimpleIndexParameters) idx.Parameters).FileName,
                "Parameter is incorrect");
        }

        /// <summary>
        /// Tests set-indexer given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerSetNullName()
        {
            source[null] = new IndexInfo("a", "b", new SimpleIndexParameters("c"));
        }

        /// <summary>
        /// Tests set-indexer given empty index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerSetEmpty()
        {
            source[" "] = new IndexInfo("a", "b", new SimpleIndexParameters("c"));
        }

        /// <summary>
        /// Tests set-indexer given null IndexInfo.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerSetNullIndexInfo()
        {
            source["testNull"] = null;
        }

        /// <summary>
        /// Tests set-indexer given existing index name.
        /// It should replace existing entry.
        /// </summary>
        [Test]
        public void TestIndexerSetExist()
        {
            source["Doc0"] = new IndexInfo(assemblyName, className, new SimpleIndexParameters("c"));
            IndexInfo idx = source["Doc0"];
            Assert.AreEqual(assemblyName, idx.AssemblyName, "AssemblyName is incorrect");
            Assert.AreEqual(className, idx.ClassName, "ClassName is incorrect");
            Assert.AreEqual("c", ((SimpleIndexParameters) idx.Parameters).FileName,
                "Parameter is incorrect");
        }

        /// <summary>
        /// Tests set-indexer given IndexInfo with some arbitrary AssemblyName.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerAssembly()
        {
            source["Doc"] = new IndexInfo("aa", className, new SimpleIndexParameters("c"));
        }

        /// <summary>
        /// Tests set-indexer given IndexInfo with some arbitrary ClassName.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerClass()
        {
            source["Doc"] = new IndexInfo(assemblyName, "cc", new SimpleIndexParameters("c"));
        }

        /// <summary>
        /// Tests set-indexer given IndexInfo with non SimpleIndexParameters.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerNonSimpleIndexParameters()
        {
            source["Doc"] =  new IndexInfo(assemblyName, className, new MyParameters());
        }

        /// <summary>
        /// Tests indexer trims the index name.
        /// </summary>
        [Test]
        public void TestIndexerTrim()
        {
            // create a new index info, add with name that contains spaces
            IndexInfo idx = new IndexInfo(assemblyName, className, new SimpleIndexParameters("file"));
            source[" Trim Test  "] = idx;

            IndexInfo assert = source["Trim Test"];
            Assert.AreEqual(assemblyName, assert.AssemblyName, "AssemblyName is incorrect");
            Assert.AreEqual(className, assert.ClassName, "ClassName is incorrect");
            Assert.AreEqual("file", ((SimpleIndexParameters) assert.Parameters).FileName,
                "Parameter is incorrect");
        }

        /// <summary>
        /// Tests Add accuracy by adding a new index info (name is case-sensitive).
        /// </summary>
        [Test]
        public void TestAdd()
        {
            // create a new index info, add with the name doc0
            IndexInfo idx = new IndexInfo(assemblyName, className, new SimpleIndexParameters("file"));
            source.Add("doc0", idx);

            IndexInfo assert = source["doc0"];
            Assert.AreEqual(assemblyName, idx.AssemblyName, "AssemblyName is incorrect");
            Assert.AreEqual(className, idx.ClassName, "ClassName is incorrect");
            Assert.AreEqual("file", ((SimpleIndexParameters) idx.Parameters).FileName,
                "Parameter is incorrect");
        }

        /// <summary>
        /// Tests Add trim the word.
        /// </summary>
        [Test]
        public void TestAddTrim()
        {
            // create a new index info, add with the name that contains spaces
            IndexInfo idx = new IndexInfo(assemblyName, className, new SimpleIndexParameters("file"));
            source.Add(" Trim Test  ", idx);

            IndexInfo assert = source["Trim Test"];
            Assert.AreEqual(assemblyName, assert.AssemblyName, "AssemblyName is incorrect");
            Assert.AreEqual(className, assert.ClassName, "ClassName is incorrect");
            Assert.AreEqual("file", ((SimpleIndexParameters) assert.Parameters).FileName,
                "Parameter is incorrect");
        }

        /// <summary>
        /// Tests Add given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddNullName()
        {
            source.Add(null, new IndexInfo("a", "b", new SimpleIndexParameters("c")));
        }

        /// <summary>
        /// Tests Add given empty index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddEmpty()
        {
            source.Add(" ", new IndexInfo("a", "b", new SimpleIndexParameters("c")));
        }

        /// <summary>
        /// Tests Add given null IndexInfo.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddNullIndexInfo()
        {
            source.Add("testNull", null);
        }

        /// <summary>
        /// Tests Add given existing index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddExist()
        {
            source.Add("Doc1", new IndexInfo("a", "b", new SimpleIndexParameters("c")));
        }

        /// <summary>
        /// Tests Add given IndexInfo with some arbitrary AssemblyName.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddAssembly()
        {
            source.Add("Doc", new IndexInfo("aa", className, new SimpleIndexParameters("c")));
        }

        /// <summary>
        /// Tests Add given IndexInfo with some arbitrary ClassName.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddClass()
        {
            source.Add("Doc", new IndexInfo(assemblyName, "cc", new SimpleIndexParameters("c")));
        }

        /// <summary>
        /// Tests Add given IndexInfo with non SimpleIndexParameters.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddNonSimpleIndexParameters()
        {
            source.Add("Doc", new IndexInfo(assemblyName, className, new MyParameters()));
        }

        /// <summary>
        /// Tests Remove accuracy by removing existing name.
        /// </summary>
        [Test]
        public void TestRemove()
        {
            Assert.IsNotNull(source["Doc0"], "Doc0 before removed");
            source.Remove("Doc0");
            Assert.IsNull(source["Doc0"], "Doc0 is removed");
        }

        /// <summary>
        /// Tests Remove by removing non-existing name (name is case-sensitive).
        /// Nothing should be removed.
        /// </summary>
        [Test]
        public void TestRemoveNotExist()
        {
            Assert.IsNotNull(source["Doc0"], "Doc0 exists");
            Assert.IsNotNull(source["Doc1"], "Doc1 exists");
            source.Remove("doc0");
            Assert.IsNotNull(source["Doc0"], "Doc0 still exists");
            Assert.IsNotNull(source["Doc1"], "Doc1 still exists");
        }

        /// <summary>
        /// Tests Remove given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestRemoveNull()
        {
            source.Remove(null);
        }

        /// <summary>
        /// Tests Remove given an empty string index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestRemoveEmpty()
        {
            source.Remove(" ");
        }

        /// <summary>
        /// Tests GetEnumerator accuracy by iterating existing collection.
        /// </summary>
        [Test]
        public void TestGetEnumerator()
        {
            IDictionaryEnumerator it = source.GetEnumerator();

            // 1st entry
            Assert.IsTrue(it.MoveNext(), "1st entry");
            Assert.AreEqual("Doc0", it.Key, "1st key");
            Assert.AreEqual(files[0], ((SimpleIndexParameters) ((IndexInfo) it.Value).Parameters).FileName,
                "1st parameter file name");

            // 2nd entry
            Assert.IsTrue(it.MoveNext(), "2nd entry");
            Assert.AreEqual("Doc1", it.Key, "2nd key");
            Assert.AreEqual(files[1], ((SimpleIndexParameters) ((IndexInfo) it.Value).Parameters).FileName,
                "2nd parameter file name");

            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Add and GetEnumerator accuracy that a new element is added.
        /// </summary>
        [Test]
        public void TestGetEnumeratorAdd()
        {
            source.Add("Doc2", new IndexInfo(assemblyName, className, new SimpleIndexParameters("cc")));
            IDictionaryEnumerator it = source.GetEnumerator();

            for (int i = 0; i < 3; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Entry: " + i);
                Assert.AreEqual("Doc" + i, it.Key, "Key: " + i);
            }
            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Remove and GetEnumerator accuracy that an element is removed.
        /// </summary>
        [Test]
        public void TestGetEnumeratorRemove()
        {
            source.Remove("Doc1");
            IDictionaryEnumerator it = source.GetEnumerator();

            for (int i = 0; i < 1; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Entry: " + i);
                Assert.AreEqual("Doc" + i, it.Key, "Key: " + i);
            }
            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Contains accuracy by checking existing and non-existing name.
        /// </summary>
        [Test]
        public void TestContains()
        {
            Assert.IsTrue(source.Contains("Doc0"), "1st entry");
            Assert.IsTrue(source.Contains("Doc1"), "2nd entry");
            Assert.IsFalse(source.Contains("doc1"), "Name is case-sensitive");
        }

        /// <summary>
        /// Tests Contains given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestContainsNull()
        {
            source.Contains(null);
        }

        /// <summary>
        /// Tests Contains given an empty string index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestContainsEmpty()
        {
            source.Contains(" ");
        }

        /// <summary>
        /// Tests Clear removes all collections.
        /// Assert that it is cleared via Contains, indexer, and GetEnumerator.
        /// </summary>
        [Test]
        public void TestClear()
        {
            source.Clear();
            Assert.IsFalse(source.Contains("Doc0"), "No 1st entry");
            Assert.IsFalse(source.Contains("Doc1"), "No 2nd entry");
            Assert.IsNull(source["Doc0"], "No 1st index");
            Assert.IsNull(source["Doc1"], "No 2nd index");

            Assert.IsFalse(source.GetEnumerator().MoveNext(), "No entry in enumerator");
        }

        /// <summary>
        /// Tests Load accuracy by loading an existing file.
        /// </summary>
        [Test]
        public void TestLoad()
        {
            // load sample collection file
            source.Load(new SimpleIndexParameters(colFile));
            IDictionaryEnumerator it = source.GetEnumerator();

            // 3 records should be loaded
            Assert.IsTrue(it.MoveNext(), "1st data is loaded");
            Assert.IsTrue(it.MoveNext(), "2nd data is loaded");
            Assert.IsTrue(it.MoveNext(), "3rd data is loaded");
            Assert.IsFalse(it.MoveNext(), "No more data");

            // assert data
            for (int i = 0; i < 3; i++)
            {
                IndexInfo idx = source["Document " + i];
                Assert.IsTrue(idx.AssemblyName.StartsWith(assemblyName),
                    "AssemblyName: " + i + ": " + idx.AssemblyName);
                Assert.AreEqual(className, idx.ClassName, "ClassName: " + i);
                Assert.AreEqual(@"..\..\test_files\Document" + i + ".xml",
                    ((SimpleIndexParameters) idx.Parameters).FileName, "Parameter: " + i);
            }
        }

        /// <summary>
        /// Tests Load given null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestLoadNull()
        {
            source.Load(null);
        }

        /// <summary>
        /// Tests Load given Parameter which is not SimpleIndexParameters.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadNotSimpleIndexParameters()
        {
            source.Load(new MyParameters());
        }

        /// <summary>
        /// Tests Load given Parameter that points to a directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadDirectory()
        {
            source.Load(new SimpleIndexParameters(@"..\..\test_files"));
        }

        /// <summary>
        /// Tests Load given Parameter that points to the current directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadDirectoryCurrent()
        {
            source.Load(new SimpleIndexParameters("."));
        }

        /// <summary>
        /// Tests Load given Parameter that points to the parent directory.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadDirectoryParent()
        {
            source.Load(new SimpleIndexParameters(".."));
        }

        /// <summary>
        /// Tests Load given Parameter that points to a malformed path that contains a &gt; char.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadMalformedIllegalChar()
        {
            source.Load(new SimpleIndexParameters(@"..\..\test_files\malformed>name"));
        }

        /// <summary>
        /// Tests Load given Parameter that points to a very long path (malformed).
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadMalformedLong()
        {
            // build a very long file name
            StringBuilder path = new StringBuilder();
            path.Append('a', 100000);
            source.Load(new SimpleIndexParameters(path.ToString()));
        }

        /// <summary>
        /// Tests Load given Parameter that points to a non-existing file.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadNotExist()
        {
            source.Load(new SimpleIndexParameters(@"..\..\test_files\NotExists.xml"));
        }

        /// <summary>
        /// Tests Load given an empty file (missing root element).
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadEmptyFile()
        {
            base.WriteToFile("", colFile);
            source.Load(new SimpleIndexParameters(colFile));
        }

        /// <summary>
        /// Tests Load given a file with invalid root element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadInvalidRoot()
        {
            base.WriteToFile("<Docs/>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
        }

        /// <summary>
        /// Tests Load given a file with invalid Document element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadInvalidDocument()
        {
            base.WriteToFile("<Documents><Doc/></Documents>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
        }

        /// <summary>
        /// Tests Load given a file with empty Document element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadEmptyDocument()
        {
            base.WriteToFile("<Documents><Document/></Documents>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
        }

        /// <summary>
        /// Tests Load given a file with duplicate Name element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadDuplicateName()
        {
            base.WriteToFile("<Documents><Document><Name>x</Name><FilePath>y</FilePath>"
                + "<Name>z</Name></Document></Documents>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
        }

        /// <summary>
        /// Tests Load given a file with missing Name element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadMissingName()
        {
            base.WriteToFile("<Documents><Document><FilePath>y</FilePath></Document></Documents>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
        }

        /// <summary>
        /// Tests Load given a file with missing FilePath element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadMissingFilePath()
        {
            base.WriteToFile("<Documents><Document><Name>x</Name></Document></Documents>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
        }

        /// <summary>
        /// Tests Load given a file with empty Name element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadEmptyName()
        {
            base.WriteToFile("<Documents><Document><Name> </Name>"
                + "<FilePath>y</FilePath></Document></Documents>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
        }

        /// <summary>
        /// Tests Load given a file with empty FilePath element.
        /// Expecting LoadIndexDataException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(LoadIndexDataException))]
        public void TestLoadEmptyFilePath()
        {
            base.WriteToFile("<Documents><Document><Name>x</Name>"
                + "<FilePath> </FilePath></Document></Documents>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
        }

        /// <summary>
        /// Tests Load preserves the existing data if any exception occurs (transactionality test).
        /// </summary>
        [Test]
        public void TestLoadTransact()
        {
            // load an invalid file (cause exception)
            try
            {
                base.WriteToFile("<Documents><Document>"
                    + "<Name>x</Name><FilePath> </FilePath></Document></Documents>", colFile);
                source.Load(new SimpleIndexParameters(colFile));
                Assert.Fail("LoadIndexDataException expected");
            }
            catch (LoadIndexDataException)
            {
                // ok
            }

            // make sure existing data is preserved
            // 1st index info
            IndexInfo idx = source["Doc0"];
            Assert.AreEqual(assemblyName, idx.AssemblyName, "First AssemblyName is incorrect");
            Assert.AreEqual(className, idx.ClassName, "First ClassName is incorrect");
            Assert.AreEqual(files[0], ((SimpleIndexParameters) idx.Parameters).FileName,
                "First Parameter is incorrect");

            // 2nd index info
            idx = source["Doc1"];
            Assert.AreEqual(assemblyName, idx.AssemblyName, "First AssemblyName is incorrect");
            Assert.AreEqual(className, idx.ClassName, "First ClassName is incorrect");
            Assert.AreEqual(files[1], ((SimpleIndexParameters) idx.Parameters).FileName,
                "First Parameter is incorrect");
        }

        /// <summary>
        /// Tests Load given a file containing an empty root.
        /// </summary>
        [Test]
        public void TestLoadEmpty()
        {
            base.WriteToFile("<Documents/>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
            Assert.IsFalse(source.GetEnumerator().MoveNext(), "No data is loaded");
        }

        /// <summary>
        /// Tests Load given a file containing &lt; &gt; chars in the name/filepath.
        /// </summary>
        [Test]
        public void TestLoadEscapedChars()
        {
            // whitespaces should also be trimmed
            base.WriteToFile("<Documents><Document><Name> &lt;  </Name>"
                + "<FilePath> &gt;  </FilePath></Document></Documents>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
            Assert.AreEqual(">", ((SimpleIndexParameters) source["<"].Parameters).FileName, "Escaped char");
        }

        /// <summary>
        /// Tests Load given a file containing CDATA tag in the name/filepath.
        /// </summary>
        [Test]
        public void TestLoadCData()
        {
            base.WriteToFile("<Documents><Document><Name><![CDATA[<Test/>]]></Name>"
                + "<FilePath><![CDATA[<Path>]]></FilePath></Document></Documents>", colFile);
            source.Load(new SimpleIndexParameters(colFile));
            Assert.AreEqual("<Path>", ((SimpleIndexParameters) source["<Test/>"].Parameters).FileName, "CData");
        }

        /// <summary>
        /// Tests Load resets the FilePath.
        /// </summary>
        [Test]
        public void TestLoadResetFilePath()
        {
            // load should reassign the FilePath
            // so we change FilePath temporarily to make sure it'd be reset
            source.FilePath = files[0];
            Assert.AreEqual(files[0], source.FilePath, "The FilePath is changed");
            source.Load(new SimpleIndexParameters(colFile));
            Assert.AreEqual(colFile, source.FilePath, "The FilePath is reset");
        }

        /// <summary>
        /// Tests Save accuracy by saving the XmlCollectionIndexSource initialized in SetUp().
        /// </summary>
        [Test]
        public void TestSave()
        {
            // save to sample collection file
            source.Save();

            string actual = base.ReadContent(colFile, true);
            Assert.AreEqual(@"<?xml version=""1.0"" encoding=""utf-8""?>"
                + @"<Documents>"
                    + @"<Document><Name>Doc0</Name><FilePath>..\..\test_files\Doc0.xml</FilePath></Document>"
                    + @"<Document><Name>Doc1</Name><FilePath>..\..\test_files\Doc1.xml</FilePath></Document>"
                + @"</Documents>", actual, "Written xml is incorrect");

            // reloading the file should not raise exception
            source.Load(new SimpleIndexParameters(source.FilePath));
        }

        /// <summary>
        /// Tests Save throws InvalidOperationException
        /// if no FilePath is specified yet.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestSaveNull()
        {
            // blank FilePath initially
            source = new XmlCollectionIndexSource();
            source.Save();
        }

        /// <summary>
        /// Tests Save given empty collection.
        /// </summary>
        [Test]
        public void TestSaveEmpty()
        {
            source.Clear();
            source.Save();
            string actual = base.ReadContent(colFile, true);
            Assert.AreEqual(@"<?xml version=""1.0"" encoding=""utf-8""?>"
                + @"<Documents />", actual, "Written xml is incorrect");

            // reloading the file should not raise exception
            source.Load(new SimpleIndexParameters(source.FilePath));
        }

        /// <summary>
        /// Tests Save given collection with characters that must be escaped.
        /// </summary>
        [Test]
        public void TestSaveEscapedChars()
        {
            source.Clear();
            source.Add("<Test />", new IndexInfo(assemblyName, className, new SimpleIndexParameters("</Path>")));
            source.Save();
            string actual = base.ReadContent(colFile, true);
            Assert.AreEqual(@"<?xml version=""1.0"" encoding=""utf-8""?>"
                + @"<Documents>"
                    + @"<Document><Name>&lt;Test /&gt;</Name><FilePath>&lt;/Path&gt;</FilePath></Document>"
                + @"</Documents>", actual, "Written xml is incorrect");

            // reloading the file should not raise exception
            source.Load(new SimpleIndexParameters(source.FilePath));
        }

        /// <summary>
        /// Tests Save and Load.
        /// </summary>
        [Test]
        public void TestSaveAndLoad()
        {
            // create 2 indices and save
            source.Clear();
            source.Add("<Doc1/>", new IndexInfo(assemblyName, className, new SimpleIndexParameters("Path1")));
            source.Add("Doc2>", new IndexInfo(assemblyName, className, new SimpleIndexParameters("<Path2 />")));
            source.Save();

            // clear and reload
            source.Clear();
            Assert.IsFalse(source.GetEnumerator().MoveNext(), "Data cleared");
            source.Load(new SimpleIndexParameters(colFile));

            // assert 1st doc
            IndexInfo idx = source["<Doc1/>"];
            Assert.IsTrue(idx.AssemblyName.StartsWith(assemblyName), "1st AssemblyName is corrected");
            Assert.AreEqual(className, idx.ClassName, "1st ClassName is corrected");
            Assert.AreEqual("Path1", ((SimpleIndexParameters) idx.Parameters).FileName,
                "Doc1 is not reloaded");

            // assert 2nd doc
            idx = source["Doc2>"];
            Assert.IsTrue(idx.AssemblyName.StartsWith(assemblyName), "2nd AssemblyName is corrected");
            Assert.AreEqual(className, idx.ClassName, "2nd ClassName is corrected");
            Assert.AreEqual("<Path2 />", ((SimpleIndexParameters) idx.Parameters).FileName,
                "Doc2 is not reloaded");
        }
    }
}
